Class: Zizq::Resources::CronGroup
- Defined in:
- lib/zizq/resources/cron_group.rb
Overview
Typed wrapper around a cron group response hash.
Instance Attribute Summary
Attributes inherited from Resource
Instance Method Summary collapse
-
#entries ⇒ Object
Returns the entries in this group as typed resources.
-
#name ⇒ Object
: () -> String.
-
#paused ⇒ Object
: () -> bool.
-
#paused? ⇒ Boolean
: () -> bool.
-
#paused_at ⇒ Object
: () -> Float?.
-
#resumed_at ⇒ Object
: () -> Float?.
Methods inherited from Resource
Constructor Details
This class inherits a constructor from Zizq::Resources::Resource
Instance Method Details
#entries ⇒ Object
Returns the entries in this group as typed resources.
18 19 20 |
# File 'lib/zizq/resources/cron_group.rb', line 18 def entries #: () -> Array[CronEntry] (@data["entries"] || []).map { |e| CronEntry.new(client, e) } end |
#name ⇒ Object
: () -> String
11 |
# File 'lib/zizq/resources/cron_group.rb', line 11 def name = @data["name"] #: () -> String |
#paused ⇒ Object
: () -> bool
12 |
# File 'lib/zizq/resources/cron_group.rb', line 12 def paused = @data["paused"] #: () -> bool |
#paused? ⇒ Boolean
: () -> bool
13 |
# File 'lib/zizq/resources/cron_group.rb', line 13 def paused? = paused #: () -> bool |
#paused_at ⇒ Object
: () -> Float?
14 |
# File 'lib/zizq/resources/cron_group.rb', line 14 def paused_at = ms_to_seconds(@data["paused_at"]) #: () -> Float? |
#resumed_at ⇒ Object
: () -> Float?
15 |
# File 'lib/zizq/resources/cron_group.rb', line 15 def resumed_at = ms_to_seconds(@data["resumed_at"]) #: () -> Float? |