Module: Zizq::_JobClass

Defined in:
sig/zizq.rbs

Overview

Structural type for a class that includes Zizq::Job.

This serves only to ensure the class can be instantiated without args.

The _ prefix is mandatory RBS syntax for interfaces. The job_class type alias below provides a cleaner name for use in source annotations.

Instance Method Summary collapse

Instance Method Details

#newJob

Returns:



92
# File 'sig/zizq.rbs', line 92

def new: () -> Job