Class: Zizq::TlsConfiguration
- Inherits:
-
Struct
- Object
- Struct
- Zizq::TlsConfiguration
- Defined in:
- lib/zizq/tls_configuration.rb
Overview
TLS settings for connecting to the Zizq server over HTTPS.
Set inside a ‘Zizq.configure` block via the `c.tls` accessors:
Zizq.configure do |c|
c.tls.ca = "/path/to/ca-cert.pem"
c.tls.client_cert = "/path/to/client-cert.pem"
c.tls.client_key = "/path/to/client-key.pem"
end
All values may be PEM-encoded strings or file paths.
Note: Mutual TLS support requires a Zizq Pro license on the server.
Instance Attribute Summary collapse
-
#ca ⇒ Object
Returns the value of attribute ca.
-
#client_cert ⇒ Object
Returns the value of attribute client_cert.
-
#client_key ⇒ Object
Returns the value of attribute client_key.
Instance Attribute Details
#ca ⇒ Object
Returns the value of attribute ca
21 22 23 |
# File 'lib/zizq/tls_configuration.rb', line 21 def ca @ca end |
#client_cert ⇒ Object
Returns the value of attribute client_cert
21 22 23 |
# File 'lib/zizq/tls_configuration.rb', line 21 def client_cert @client_cert end |
#client_key ⇒ Object
Returns the value of attribute client_key
21 22 23 |
# File 'lib/zizq/tls_configuration.rb', line 21 def client_key @client_key end |